#
# mygetopt.rb
#

require 'getoptlong'

class MyGetoptLong < GetoptLong

  def MyGetoptLong.new( usage, descripter )
    parser = super(*descripter.map {|line|
      next if line.strip.empty?
      display, shortopt, longopt, optarg, doc = line.split
      a = []
      a.push longopt unless longopt == '-'
      a.push shortopt unless shortopt == '-'
      a.push optarg == '-' ? GetoptLong::NO_ARGUMENT : GetoptLong::REQUIRED_ARGUMENT
      a
    }.compact)
    parser.quiet = true
    parser.instance_eval {
      @usage = usage
      @descripter = descripter.strip
    }
    parser
  end

  def usage( status, errmsg = nil )
    f = (status == 0 ? $stdout : $stderr)
    f.puts "#{File.basename $0}: #{errmsg}" if errmsg
    @usage.each do |uline|
      if /%%options%%/ === uline
        print_options f
      else
        f.print uline
      end
    end
    exit status
  end

  def print_options( f )
    @descripter.each do |line|
      if line.strip.empty?
        f.puts
        next
      end
      display, shortopt, longopt, optarg, doc = line.split
      next unless display
      opt = [ [ennil(shortopt), ennil(longopt)].compact.join(','),
              ennil(optarg) ].compact.join(' ')
      f.printf "  %-27s %s\n", opt, doc
    end
  end

  def ennil(opt)
    opt == '-' ? nil : opt
  end

end
